#
# spec file for package ntpd-rs
#
# Copyright (c) 2025 SUSE LLC
# Copyright (c) 2024-2025, Martin Hauke <mardnh@gmx.de>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define services ntpd-rs.service ntpd-rs-metrics.service
Name:           ntpd-rs
Version:        1.5.0
Release:        1.1
Summary:        Full-featured implementation of NTP with NTS support
License:        Apache-2.0 OR MIT
Group:          Productivity/Networking/Other
#Git-Clone:     https://github.com/pendulum-project/ntpd-rs
URL:            https://tweedegolf.nl/en/pendulum
Source:         https://github.com/pendulum-project/ntpd-rs/archive/refs/tags/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Source1:        vendor.tar.zst
Source2:        %{name}.tmpfiles
Source3:        %{name}.sysusers
BuildRequires:  cargo
BuildRequires:  cargo-packaging
BuildRequires:  systemd-rpm-macros
BuildRequires:  sysuser-tools
Requires(pre):  %{name}-common = %{version}
Conflicts:      ntp-daemon
Provides:       ntp-daemon

%description
A full-featured implementation of the Network Time Protocol,
including support for NTS (Network Time Security).

It includes both client and server support.

%package common
Summary:        System user 'ntpd-rs' and 'ntpd-rs-observe'
BuildArch:      noarch
%sysusers_requires

%description common
A full-featured implementation of the Network Time Protocol,
including support for NTS (Network Time Security).

This subpackage sets up the system user/group for the rest of ntpd-rs.

%prep
%autosetup -p 1 -a 1

%build
%{cargo_build}
%sysusers_generate_pre %{SOURCE3} %{name} %{name}.conf

%install
install -Dpm0755 target/release/ntp-ctl -t %{buildroot}/%{_bindir}
install -Dpm0755 target/release/ntp-daemon -t %{buildroot}/%{_bindir}
install -Dpm0755 target/release/ntp-metrics-exporter -t %{buildroot}/%{_bindir}

install -Dpm0644 docs/precompiled/man/*.5 -t %{buildroot}/%{_mandir}/man5
install -Dpm0644 docs/precompiled/man/*.8 -t %{buildroot}/%{_mandir}/man8

install -Dpm0644 docs/examples/conf/ntp.toml.default %{buildroot}/%{_sysconfdir}/ntpd-rs/ntp.toml

install -Dpm0644 docs/examples/conf/ntpd-rs.service -t %{buildroot}/%{_unitdir}
install -Dpm0644 docs/examples/conf/ntpd-rs-metrics.service -t %{buildroot}/%{_unitdir}

install -Dpm0644 %{SOURCE2} %{buildroot}/%{_tmpfilesdir}/%{name}.conf
install -Dpm0644 %{SOURCE3} %{buildroot}%{_sysusersdir}/%{name}.conf

install -d %{buildroot}/%{_sharedstatedir}/ntpd-rs
install -d %{buildroot}/%{_sharedstatedir}/ntpd-rs-observe
install -d %{buildroot}/%{_rundir}/ntpd-rs

%pre
%service_add_pre %{services}

%pre common -f %{name}.pre

%post
%service_add_post %{services}
systemd-tmpfiles --create %{_tmpfilesdir}/%{name}.conf || true

%preun
%service_del_preun %{services}

%postun
%service_del_postun %{services}

%check
### skip tests that are not working in OBS build environment
#[  147s] running 2 tests
#[  147s] test hwtimestamp::tests::get_hwtimestamp ... FAILED
#[  147s] test socket::tests::test_send_timestamp ... FAILED
#%%{cargo_test} -- -- --skip hwtimestamp::tests::get_hwtimestamp --skip socket::tests::test_send_timestamp

%files
%license COPYRIGHT LICENSE-APACHE LICENSE-MIT
%doc CHANGELOG.md README.md SECURITY.md
%doc docs/examples/conf/ntp.toml.default
%dir %{_sysconfdir}/ntpd-rs
%config(noreplace) %{_sysconfdir}/ntpd-rs/ntp.toml
%{_bindir}/ntp-ctl
%{_bindir}/ntp-daemon
%{_bindir}/ntp-metrics-exporter
%{_mandir}/man5/ntp.toml.5%{?ext_man}
%{_mandir}/man8/ntp-ctl.8%{?ext_man}
%{_mandir}/man8/ntp-daemon.8%{?ext_man}
%{_mandir}/man8/ntp-metrics-exporter.8%{?ext_man}
%{_tmpfilesdir}/%{name}.conf
%{_unitdir}/ntpd-rs.service
%{_unitdir}/ntpd-rs-metrics.service
%ghost %dir %attr(0700, ntpd-rs, ntpd-rs) %{_sharedstatedir}/ntpd-rs
%ghost %dir %attr(0700, ntpd-rs-observe, ntpd-rs-observe) %{_sharedstatedir}/ntpd-rs-observe
%ghost %dir %attr(0700, ntpd-rs-observe, ntpd-rs-observe) %{_rundir}/ntpd-rs

%files common
%{_sysusersdir}/%{name}.conf

%changelog
* Fri Feb 28 2025 Martin Hauke <mardnh@gmx.de>
- Update to version 1.5.0
  Added
  * Support for PPS based sources (this support can be disabled
    with a compile flag for now).
  * Per source configuration of poll intervals.
  * Allow setting a custom reference id for stratum 1 servers.
  Changed
  * Our algorithm can now handle periodic sources.
  * ntpd-rs runs single-threaded when only configured as a client,
    servers still run multithreaded.
  * The reference timestamp field is now set to the truncated
    receive timestamp instead of being zero.
  * Support of rustls 0.21 and 0.22 is removed.
  * Certificate validation is now done through
    rustls-platform-verifier, following platform certificate
    validation more closely.
  * Updated dependencies
  Fixed
  * Fixed parsing of IPv6 addresses.
  * Fixed incorrect display of date in force-sync command.
  * Fixed a client denial of service vulnerability with zero-sized
    NTS cookies.
  * Fixed a client denial of service vulnerability with NTS cookies
    that are too large.
* Fri Dec 13 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.4.0
  Added
  * Support for GPS based sources via a GPSd socket.
  * Added a setting to allow disabling of colors in our logs
    (observability.ansi-colors)
  Changed
  * Fallback to V4 should V5 not work for some reason (if NTPv5 is
    enabled)
  * Make the NTP version of a source configurable.
  * We now support rustls from 0.21 and up, to allow compilation
    on more targets.
  Fixed
  * Force sync did not work when a single source was configured.
  * Fix incorrect indexing in decode of ReferenceIdRequest for
    NTPv5 messages.
* Thu Nov 28 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.3.1
  * Updated dependencies. Includes fixes for RUSTSEC-2024-0399.
* Thu Sep 19 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.3.0
  Added
  * Added force-sync command to ntp-ctl to help with getting a
    decent initial time if the clock is far away from reality.
  * Added information on NTS to the ntp-ctl status overview.
  Changed
  * Made the logs a little less chatty.
  * Updated dependencies.
* Mon Jul 22 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.2.3
  Fixed
  * Metrics exporter should wait for an actual request before
    sending a response.
* Thu Jul 18 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.2.2
  Fixed
  * Fixed a bug in leap second flag handling where previous flags
    weren't explicitly unset.
  * Fixed a bug that caused NTS-KE sessions from clients with a
    large request to hang.
  * Fixed a bug that caused NTS-KE error records never to be sent.
- Update to version 1.2.1
  Fixed
  * Fixed a bug in leap second flag handling where previous flags
    weren't explicitly unset.
* Thu Jul 11 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.2.0
  Added
  * Pool mode sources can be set up to ignore some addresses
    returned by the pool.
  * NTP sources use a random client port for every request sent.
  * The metrics exporter will keep trying to startup while its
    network interface is not yet available.
  * Added option for server to only respond to NTS messages.
  Changed
  * Updated dependencies.
  * ntp-udp and ntp-os-clock were replaced by timestamped-socket
    and clock-steering.
  * Minor improvements to NTS keyset storage errors.
  * Loop detection is handled globally instead of per source.
  * The MSRV was raised to 1.70.
  * The metrics exporter is better able to handle multiple
    simultaneous connections.
  * Pendulum is now part of Trifecta Tech Foundation (copyright
    was updated).
  * Large parts of the daemon code have been refactored to
    simplify the code.
  Fixed
  * Metrics would not synchronize if the metrics data was too large
  * ntpd-rs would ignore responses from servers implementing NTPv3
* Sat Jun 29 2024 Martin Hauke <mardnh@gmx.de>
- Update to version 1.1.3
  Fix CVE-2024-38528
  * Unlimited number of NTS-KE connections could crash ntpd-rs
    server.
- Drop patch
  * 0001-Move-default-socket-path-from-var-run-to-run.patch
* Tue Apr 16 2024 Martin Hauke <mardnh@gmx.de>
- Initial package, version 1.1.2
- Add patch:
  * 0001-Move-default-socket-path-from-var-run-to-run.patch
